#define LIGHTING
#define SPECULAR

// Attributes
attribute vec4 a_position;	// Vertex position							(x, y, z, w)
attribute vec3 a_normal;	// Vertex normal							(x, y, z)
attribute vec2 a_texCoord;	// Vertex texture coordinate				(u, v)


// Uniforms
uniform mat4 u_worldMatrix;
uniform mat4 u_viewMatrix;
uniform mat4 u_projectionMatrix;

uniform mat4 u_worldViewProjectionMatrix;					// Matrix to transform a position to clip space
uniform mat4 u_inverseTransposeWorldViewMatrix;				// Matrix to transform a normal to view space
#if defined(SPECULAR) || defined(SPOT_LIGHT) || defined(POINT_LIGHT)
uniform mat4 u_worldViewMatrix;		// Matrix to tranform a position to view space
#endif
#if defined(SKINNING)
uniform vec4 u_matrixPalette[SKINNING_JOINT_COUNT * 3];		// Array of 4x3 matrices
#endif
#if defined(SPECULAR)
uniform vec3 u_cameraPosition;                 				// Position of the camera in view space
#endif



// Varyings
varying vec3 v_normalVector;	// Normal vector in view space
varying vec2 v_texCoord;	// Texture coordinate
#if defined(SPECULAR)
varying vec3 v_cameraDirection;								// Direction the camera is looking at in tangent space
#endif
#if defined(POINT_LIGHT)
varying vec3 v_vertexToPointLightDirection;					// Direction of point light w.r.t current vertex in tangent space
varying float v_pointLightAttenuation;						// Attenuation of point light
#include "lighting-point.vert"
#elif defined(SPOT_LIGHT)
varying vec3 v_vertexToSpotLightDirection;					// Direction of the spot light w.r.t current vertex in tangent space
varying float v_spotLightAttenuation;						// Attenuation of spot light

// Lighting
#include "lighting-spot.vert"
#else
#include "lighting-directional.vert"
#endif

#include "skinning-none.vert" 



void main() {
	v_texCoord = a_texCoord;

    // Get the position and normal
    vec4 position = getPosition();
    vec3 normal = getNormal();

	vec4 vPos = u_worldViewMatrix * position;

	float id = -position.z;
	float ang = id;

	vec4 wPos = vec4(cos(ang), id*0.1, sin(ang), 1.0);
	vec4 viewPos = u_viewMatrix * wPos;
	float side = 0.5;
	viewPos.xy += position.xy * side;
	vec4 pPos = u_projectionMatrix * viewPos;

	gl_Position = pPos;

    // Transform normal to view space.
	mat3 normalMatrix = mat3(u_inverseTransposeWorldViewMatrix[0].xyz, u_inverseTransposeWorldViewMatrix[1].xyz, u_inverseTransposeWorldViewMatrix[2].xyz);
    v_normalVector = normalMatrix * normal;

    // Apply light.
    applyLight(position);
}
